#!/bin/bash
SCRIPT_NAME=`basename $0`

SSH_HOST=192.168.0.1
SSH_USER=admin
SSH_PORT=22
REPORT_EMAIL=example@example.com
MAX_SHUTDOWN_COUNT=3

if [ -t 1 ]; then
    ANSI_RESET="$(tput sgr0)"
    ANSI_RED="$(tput bold)$(tput setaf 1)"
    ANSI_GREEN="$(tput bold)$(tput setaf 2)"
fi

while getopts "" OPT; do
    case $OPT in
        *)  echo
            echo    "  SYNOPSIS"
            echo -e "  $SCRIPT_NAME"
            echo
            echo    "  DESCRIPTION"
            echo    "  Checks UPS status repeatedly and shuts down the machine if power goes out on Mikrotik router."
            echo
            echo    "  SAMPLES"
            echo    "  $0"
            echo
            exit 255
            ;;
    esac
done
shift $((OPTIND-1))

PID_FILE="/var/run/.$SCRIPT_NAME.pid"
PID_LAST=`cat $PID_FILE 2> /dev/null`
if [ -n "$PID_LAST" ]; then
    PID_ACTIVE=`ps -ax $PID_LAST | grep "^$PID_LAST" | grep "$SCRIPT_NAME"`
    if [ -n "$PID_ACTIVE" ]; then
        echo "$SCRIPT_NAME: script is already running!" >&2
        exit 255
    fi
fi
echo $$ > $PID_FILE
trap "echo -ne '${ANSI_RESET}' ; rm $PID_FILE 2>/dev/null ; exit" EXIT SIGHUP SIGINT SIGTERM


SHUTDOWN_COUNT=0
while (true); do
    VOLTAGES=`ssh -p $SSH_PORT $SSH_USER@$SSH_HOST "/system health print" | egrep 'psu[12]-voltage' | cut -d: -f2 | tr -d '\r' | tr '\n' ' ' | xargs`
    VOLTAGES_AVAILABLE=`echo $VOLTAGES | tr ' ' '\n' | grep -v '0V' | wc -l`

    if [[ "$VOLTAGES_AVAILABLE" -lt 2 ]]; then
        SHUTDOWN_COUNT=$(( $SHUTDOWN_COUNT + 1 ))
        echo "${ANSI_RED}NOK ($SHUTDOWN_COUNT/$MAX_SHUTDOWN_COUNT): $VOLTAGES${ANSI_RESET}"
        if [[ "$SHUTDOWN_COUNT" -ge "$MAX_SHUTDOWN_COUNT" ]]; then
            echo -e "Subject: ‼ `hostname` going down ‼\n\nUPS power not restored" |& msmtp $REPORT_EMAIL
            sleep 5
            poweroff
            exit 1
        fi
    else
        echo -e "${ANSI_GREEN}OK: $VOLTAGES${ANSI_RESET}"
        SHUTDOWN_COUNT=0
    fi

    sleep 60
done
